<?php

namespace General\Helpers;

use Illuminate\Support\Facades\Log;
use OSS\Core\OssException;
use OSS\OssClient;

class AliOSSHelper
{

    private $oss_client;

    private $bucket;

    public function __construct()
    {
        $this->oss_client = new OssClient(env('OSS_ACCESS_ID'), env('OSS_ACCESS_KEY'), env('OSS_END_POINT'));
        $this->bucket = env('OSS_BUCKET', 'zhuishuyun');
    }

    public function uploadFile(string $oss_path, string $file_path, string $bucket = null)
    {
        $bucket = $bucket ? $bucket : $this->bucket;
        try {
            $oss_file = $this->oss_client->uploadFile($bucket, $oss_path, $file_path);
            return $oss_file['oss-request-url'];
        } catch (OssException $e) {
            Log::error($e->getMessage());
            return null;
        }
    }

    public function uploadString(string $oss_path, string $content, string $bucket = null)
    {
        $bucket = $bucket ? $bucket : $this->bucket;
        try {
            $oss_file = $this->oss_client->putObject($bucket, $oss_path, $content);
            return $oss_file['oss-request-url'];
        } catch (OssException $e) {
            Log::error($e->getMessage());
            return null;
        }
    }

    public function delFile(string $oss_path, string $bucket = null)
    {
        $bucket = $bucket ? $bucket : $this->bucket;
        try {
            $oss_file = $this->oss_client->deleteObject($bucket, $oss_path);
            return $oss_file['oss-request-url'];
        } catch (OssException $e) {
            Log::error($e->getMessage());
            return null;
        }
    }
}
