<?php

namespace General\Services\Config;

use Illuminate\Support\Collection;
use General\Models\Channel\Channel;
use General\Models\Channel\ChannelUser;
use General\Models\Config\ChannelConfigType;
use General\Models\Config\DistributionSelfDefineConfig;

/**
 * 
 * @property array  $WITHDRAW_CASH_AMOUNT_NO_LIMIT
 * @property array  $all_platform_charge_user_report
 */
class ConfigService
{
	/**
	 * 落地页自动过审code
	 */
	const landing_page_auto_check = 'landing_page_auto_check';

	protected $field = [];

	public function __get($name)
	{
		if (!isset($this->field[$name])) {
			$config = $this->findConfig($name);
			if ($config) {
				switch ($config->type) {
					case 1:
						$this->field[$name] = $this->getChannelIdsWithCompany($name);
						break;
					case 2:
						$this->field[$name] = $this->getChannelIdsWithAccount($name);
						break;
					case 3:
						$this->field[$name] = $this->getChannelIds($name);
						break;
				}
			} else {
				$this->field[$name] = $this->getChannelIds($name);
			}
		}
		return $this->field[$name];
	}

	/**
	 * @return ChannelConfigType
	 */
	private function findConfig(string $type)
	{
		return ChannelConfigType::where('code', $type)->first();
	}

	/**
	 * 获取查找的列名
	 * @return string 
	 */
	private function getConfigColumnName(string $type)
	{
		$config = $this->findConfig($type);
		$column_name = 'distribution_channel_id';
		if ($config) {
			switch ($config->type) {
				case 1:
					$column_name = 'company_id';
					break;
				case 2:
					$column_name = 'channel_user_id';
					break;
				case 3:
					$column_name = 'distribution_channel_id';
					break;
			}
		}
		return $column_name;
	}

	/**
	 * 通过公司获取站点
	 * @return array
	 */
	public function getChannelIdsWithCompany(string $type)
	{
		$company_ids = DistributionSelfDefineConfig::where('type', $type)->where('status', 1)->pluck('company_id')->all();
		$channel_user_ids = ChannelUser::whereIn('company_id', $company_ids)->pluck('id')->all();
		return  Channel::whereIn('channel_user_id', $channel_user_ids)->pluck('id')->all();
	}

	/**
	 * 通过渠道用户获取站点
	 * @return array
	 */
	public function getChannelIdsWithAccount(string $type)
	{
		$channel_user_ids = DistributionSelfDefineConfig::where('type', $type)->where('status', 1)->pluck('channel_user_id')->all();
		return  Channel::whereIn('channel_user_id', $channel_user_ids)->pluck('id')->all();
	}

	/**
	 * 获取站点
	 * @return array
	 */
	public function getChannelIds(string $type)
	{
		return DistributionSelfDefineConfig::where('type', $type)->where('status', 1)->pluck('distribution_channel_id')->all();
	}

	/**
	 * 查找配置项
	 * @param string $type
	 * @param int $id
	 * @return DistributionSelfDefineConfig
	 */
	public function findConfigItem(string $type, int $id)
	{
		$column_name = $this->getConfigColumnName($type);
		return DistributionSelfDefineConfig::where('type', $type)->where($column_name, $id)->first();
	}


	/**
	 * 判断是否有权限
	 * @param int $id
	 * @param string $type
	 * @return int  
	 */
	public function hasAuth(int $id, string $type)
	{
		$item = $this->findConfigItem($type, $id);
		return $item ? $item->status : 0;
	}

	/**
	 * 查找配置项
	 * @param string $type
	 * @param array $ids
	 * @return Collection
	 */
	public function findConfigItems(string $type, array $ids)
	{
		$column_name = $this->getConfigColumnName($type);
		return DistributionSelfDefineConfig::where('type', $type)->whereIn($column_name, $ids)->get();
	}

	/**
	 * 保存配置
	 * @param int $id
	 * @param string $code
	 * @param bool $is_open
	 */
	public function saveConfig($id, string $code, bool $is_open)
	{
		$keys = ['type' => $code];
		$config = $this->findConfig($code);
		if ($config) {
			switch ($config->type) {
				case 1:
					$keys['company_id'] = $id;
					break;
				case 2:
					$keys['channel_user_id'] = $id;
					break;
				case 3:
					$keys['distribution_channel_id'] = $id;
					break;
			}
		} else {
			$keys['distribution_channel_id'] = $id;
		}
		return DistributionSelfDefineConfig::updateOrCreate(
			$keys,
			['status' => $is_open ? 1 : 0]
		);
	}
}
