<?php

namespace General\Services\Report;

use General\Models\Order\Order;
use General\Models\Qapp\QappSendOrders;
use General\Models\Report\ReportUserBindRecord;

/**
 * 抖音数据上报
 */
class QappBaiduReport extends BaseReport
{
    public function __construct()
    {
        $this->register_event_type = 'register';
        $this->charge_event_type = 'ec_buy';
        $this->add_desk_event_type = 'activate';
    }

    public function getRegisterQueryParams(ReportUserBindRecord $user): array
    {
        return  [];
    }

    public function getChargeQueryParams(ReportUserBindRecord $user, float $amount): array
    {
        //百度回传链接
        $this->report_url = $user->link;

        $version = $this->getReportVersion($user->uid);

        if($version == 1){
            $this->charge_event_type = 'pay_to_read';
        }
        //替换a_type的值
        $this->getReplaceUrl('a_type',$this->charge_event_type);

        //替换a_value的值
        $this->getReplaceUrl('a_value',$amount * 100);

        //百度akey
        $akey = $this->getBaiduAkey($user->uid);

        $this->report_url .= '&sign=' . md5($this->report_url.$akey);

        return  [];
    }

    /**
     * 替换url中的操作类型和值
     * @param $type
     * @param $value
     * @return void
     */
    protected function getReplaceUrl($type, $value)
    {
        //获取$type的值
        $type_value = $this->matchItemInUrl($type,$this->report_url);

        //替换$type的值
        $this->report_url = str_replace($type.'='.$type_value,$type.'='.$value,$this->report_url);
    }

    /**
     * 获取百度akey
     * @param $uid
     * @return mixed|string
     */
    protected function getBaiduAkey($uid)
    {
        try {
            //获取派单平台配置
            $qapp_send_orders = $this->getQappSendOrders($uid);

            if(isset($qapp_send_orders->extra_config) && !empty($qapp_send_orders->extra_config)){

                $extra_config = json_decode($qapp_send_orders->extra_config,true);

                return isset($extra_config['baidu_akey']) ? $extra_config['baidu_akey'] : '';
            }
            return '';
        } catch (\Exception $e) {
            \Log::info('补传获取派单失败uid:' .$uid . 'error:' . $e->getMessage());
        }

    }

    /**
     * 获取上报版本
     * @param $uid
     * @return int
     */
    protected function getReportVersion($uid)
    {
        $qapp_send_orders = $this->getQappSendOrders($uid);

        if(isset($qapp_send_orders->report_version)){

            return $qapp_send_orders->report_version;
        }
        return 0;
    }

    /**
     * 获取派单
     * @param $uid
     * @return string
     */
    protected function getQappSendOrders($uid)
    {
        //获取派单id
        $send_order_id = Order::where('uid',$uid)->value('send_order_id');

        //获取派单平台配置
        $extra_config = QappSendOrders::where('send_order_id',$send_order_id)->first();

        return $extra_config ?? '';
    }

    /**
     * 获取查询字符串的值
     * @param string $item
     * @param string $url
     * @return mixed|string
     */
    protected function matchItemInUrl(string $item, string $url)
    {
        preg_match('/&?' . $item . '=([^&\s]+)&?/i', $url, $arr);
        if ($arr) {
            return $arr[1];
        }
        return '';
    }
}
